//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//--------------------------------------------------------------------------
//
//  Starter.C - The main subroutine call that initializes all of the calls.

#include "ownet.h"
#include <SysEvtMgr.h>
#include <Hardware.h>
#include "StarterRsc.h"
#include "SauthPalm.h"
#include "ibsha18.h"

// mode constants
enum { MD_FIND_ROVING=0, MD_READ_AUTH, MD_MONEY_VERF, MD_MONEY_CHANGE,
       MD_MONEY_WRITE, MD_READ_BACK_AUTH, MD_VERIFY_GONE };

// External Functions	
extern int  owAcquire(int,char *, char *);
extern int  owFirst(int,int,int);
extern void owRelease(int,char *);
extern void owSerialNum(int,uchar *, int);
extern int key_abort(void);
extern void ExitProg(char *, int);
extern int FindCoprocessor(TransState *);
extern int available_screen(int);
extern void reset_screen(void);
extern ulong msGettick(void);
extern int FindRoving(TransState *);
extern int ReadAuth(eCertificate *, TransState *);
extern int VerifyMoney(eCertificate *, TransState *);
extern int WriteMoney(eCertificate *, TransState *);
extern int ReadBackAuth(eCertificate *,TransState *);
extern int RovingIsPresent(TransState *ts);
extern void msDelay(int);

// local funcitons
static Word StartApplication(void);
void PrintSerialNum(int,int);
int PrintHeader(char *);
void output_status(int, char *);
int SetNewMoneyValue(eCertificate *,long);
int DebitRovingSHAiButton(int,long,TransState *);
void eCertPrint(eCertificate *, char *);

// globals 
long time_stamp;
// verbose output mode
extern int VERBOSE;

DWord PilotMain(Word cmd, char *cmdPBP, Word launchflags)
{
   	Word error;
	int portnum = 0;
	
   int ShowStatus=FALSE,LOOP,i,cnt,exit_code=EXIT_SUCCESS;
   char msg[255], port1[20], port2[20];
   long amount;
   TransState ts;
   char tmpstr[5];

   	if(cmd == sysAppLaunchCmdNormalLaunch)
   	{
      	error = StartApplication();
      	if(error) return error;
      	
   		// setup the port names
   		sprintf(port1,"%s","0");
   		sprintf(port2,"%s","1");
//two		port2[0] = 0;
		
   		VERBOSE = FALSE;
   		LOOP = FALSE;
   		
   		// print the header info 
   		if (!PrintHeader("Palm debit Version 2.00"))
      		return 0;
		
   		// attempt to acquire the 1-Wire Net
   		ts.portnum[IB_ROVING] = 0;
   		if (!owAcquire(ts.portnum[IB_ROVING],port2,msg))
//two		if(!owAcquire(ts.portnum[IB_ROVING],port1,msg))
   		{
      		ExitProg(msg,EXIT_ERROR_AQ1);
      		return 0;
      	}
      		
   		// check if CO_PROCESSOR port is not the same
   		if (port2[0] != 0)
   		{
      		ts.portnum[IB_COPR] = 1;
      		if (!owAcquire(ts.portnum[IB_COPR],port1,msg))
      		{
         		ExitProg(msg,EXIT_ERROR_AQ2);
   				owRelease(ts.portnum[IB_ROVING],msg);
   				return 0;
   			}
         		
   		}
   		else
      		ts.portnum[IB_COPR] = ts.portnum[IB_ROVING];

   		// set amount to debit (50 cents)
   		amount = 50;

   		// attempt to find co-processor
   		sprintf((char *)(&ts.copr_file[0]),"COPR");
   		if (FindCoprocessor(&ts))
   		{
      		// loop to print ROM of device found
      		cnt = sprintf(msg,"** Co-processor SHA iButton found:\n");
      		for (i = 7; i >= 0; i--)
         		cnt += sprintf(msg + cnt,"%02X",ts.copr_rom[i]);
      		cnt += sprintf(msg + cnt,"\n** Provider file: ");
      		
      		for (i = 0; i < 4; i++)
      			tmpstr[i] = ts.provider[i];
      		tmpstr[4] = 0;
         	cnt += sprintf(msg + cnt,"%s",tmpstr);
      		cnt += sprintf(msg + cnt,".%d\n",ts.provider[4]);
      		cnt += sprintf(msg + cnt,"** Money master secret page: %d\n",ts.c_mmaster_scrt);
      		cnt += sprintf(msg + cnt,"** Auth master secret page: %d\n",ts.c_amaster_scrt);
      		output_status(LV_ALWAYS,msg);
      		cnt=0;
      		cnt += sprintf(msg + cnt,"** Unique device (calculated)\n");
      		cnt += sprintf(msg + cnt,"secret page:  %d\n\n",ts.c_udevice_scrt);
      		output_status(LV_ALWAYS,msg);
      
      		// find and debit roving SHA iButtons
      		exit_code = DebitRovingSHAiButton(LOOP,-amount,&ts);
   		}
   		else
   		{
      		output_status(LV_ALWAYS,"ERROR, could not find co-processor on 1-Wire Net, end program\n");
      		exit_code = EXIT_ERROR_CO;
   		}

   		// release the 1-Wire Net(s)
   		owRelease(ts.portnum[IB_ROVING],msg);
   		if (port2[0] != 0)
      		owRelease(ts.portnum[IB_COPR],msg);

   		// end the program
   		exit(exit_code);
   		msDelay(5000);

	  }
	     
    return exit_code;   
}

//---------------------------------------------------------------------
static Word StartApplication(void)
{
   FrmGotoForm(MainForm);  
   return 0;
}


//----------------------------------------------------------------------
//  Read and print the Serial Number.
//
void PrintSerialNum(int portnum, int cnt)
{
   uchar serial_num[8];
   char printsn[17];
   char tempstr[9];
   int i;

   owSerialNum(portnum,serial_num,TRUE);

   for (i=7; i>=0; i--)
   {
   	StrPrintF(tempstr, "%2X", serial_num[i]);
   	printsn[(7-i)*2] = tempstr[6];
   	printsn[(7-i)*2+1] = tempstr[7];
   }

	WinDrawChars(printsn,16,45,40+(cnt*10));
}

//--------------------------------------------------------------------------
// Set new money amount (if possible)
//
int SetNewMoneyValue(eCertificate *ec, long amount_change)
{
   long l,money;
   int i;

   // convert balance to ulong
   l = 0;
   for (i = 2; i >= 0; i--)   
   {
      l <<= 8;
      l |= ec->balance[i];
   }

   // change by 'amount_change'
   if (l + amount_change < 0)
      return FALSE;
   else
      l += amount_change;
   money = l;

   // put back in balance
   for (i = 0; i < 3; i++)
   {
      ec->balance[i] = (uchar)(l & 0xFF);
      l >>= 8;
   }

   // set transaction ID to mine
   ec->trans_id = 0x1234;

   // also increment counter
   ec->write_cycle_cnt++;

   return TRUE;
}


//--------------------------------------------------------------------------
// Print the eCertificate
//
void eCertPrint(eCertificate *ec, char *msg)
{
   ulong ul;
   int i,mcnt=0;

   // check if can display on screen  ADDED for PALM
   if (!available_screen(6))
      reset_screen();  

   // device time to transact
   time_stamp = msGettick() - time_stamp;

   mcnt = sprintf(msg,"---------------------------\n");   
   mcnt += sprintf(msg + mcnt,"| Device: ");
 
   for (i = 6; i >=0; i--)
      mcnt += sprintf(msg + mcnt,"%02X",ec->serial_num[i]);
  
   mcnt += sprintf(msg + mcnt,"\n| User data: %s\n",ec->user_data);

   // convert balance to ulong
   ul = 0;
   for (i = 2; i >= 0; i--)   
   {
      ul <<= 8;
      ul |= ec->balance[i];
   }

   mcnt += sprintf(msg + mcnt,"| Balance: $%4.2f US\n",(float)(ul/100.0)); 
   mcnt += sprintf(msg + mcnt,"| Transaction time: %ld ms\n",time_stamp);
   mcnt += sprintf(msg + mcnt,"---------------------------\n");   
}


//--------------------------------------------------------------------------
// loop to find and debit sha iButtons
//
int DebitRovingSHAiButton(int LOOP, long amount, TransState *ts)
{
   eCertificate ec,vec;
   int debit_mode,check_present,mcnt;
   char msg[255],lastmsg[255];
   long tstamp;

   // start mode
   debit_mode = MD_FIND_ROVING;

   // loop to find and deduct from the money account on roving
   do
   {
      // assume need to check if device is present
      check_present = TRUE;

      // switch on current debit mode
      switch (debit_mode)
      {
         case MD_FIND_ROVING:
            // time stamp of start of search
            time_stamp = msGettick(); 
            if (FindRoving(ts))
            {
               reset_screen();  // ADDED for PALM
               sprintf(msg,"** Roving SHA iButton found\n");
               debit_mode = MD_READ_AUTH;
            }
            else  
               sprintf(msg,"\n** Waiting for roving SHA iButton\n");
            check_present = FALSE;
            break;

         case MD_READ_AUTH:
            if (ReadAuth(&ec,ts))
            {
               sprintf(msg,"** Read authentication complete\n");
               check_present = FALSE;
               debit_mode = MD_MONEY_VERF;
            }
            else  
               sprintf(msg,"ERROR doing read authentication\n");
            break;

         case MD_MONEY_VERF:
            // first check if correct money type 
            if (ec.code_mult != 0x8B48)
            {
               sprintf(msg,"\n** ERROR, money wrong type (code %04X)\n\n",ec.code_mult);
               check_present = FALSE;
               debit_mode = MD_VERIFY_GONE;
               break;     
            }
            // verify money valid
            if (VerifyMoney(&ec, ts))
            {
               sprintf(msg,"** Money value authenticated\n");
               check_present = FALSE;
               debit_mode = MD_MONEY_CHANGE;
            }
            else  
               sprintf(msg,"ERROR authenticating money\n");
            break;

         case MD_MONEY_CHANGE:
            // new value for the money
            if (!SetNewMoneyValue(&ec,amount))
            {
               sprintf(msg,"\n** ERROR, not enough funds available to debit\n\n");
               check_present = FALSE;
               debit_mode = MD_VERIFY_GONE;
            }
            else
            {
               sprintf(msg,"** New money value set (debit of %ld)\n",-amount);
               check_present = FALSE;
               debit_mode = MD_MONEY_WRITE;
            }
            break;

         case MD_MONEY_WRITE:
            if (WriteMoney(&ec,ts))
            {
               sprintf(msg,"** New money written\n");
               check_present = FALSE;
            }
            else  
               sprintf(msg,"ERROR writing new money value\n");
            debit_mode = MD_READ_BACK_AUTH;
            break;

         case MD_READ_BACK_AUTH:
            if (ReadBackAuth(&vec,ts))
            {
               check_present = FALSE;
               mcnt = sprintf(msg,"** Read back authentication complete\n");
   
               // check if written correclty
               if (ec.write_cycle_cnt == vec.write_cycle_cnt)
               {
                  // succes so print result
                  eCertPrint(&ec,&msg[mcnt]);
                  // make sure max block ok
                  if (LOOP)
                  {
                     // if in loop mode go directly to read auth
                     // (skip finding roving)
                     debit_mode = MD_READ_AUTH; 
                     // start the time stamp now
                     time_stamp = msGettick(); 
                  }
                  else
                     debit_mode = MD_VERIFY_GONE;
               }   
               // not written so go back to that mode
               else if (ec.write_cycle_cnt == (vec.write_cycle_cnt + 1))
                  debit_mode = MD_MONEY_WRITE;
               // unknown error value
               else
                  return EXIT_ERROR;
            }
            else  
               sprintf(msg,"** Error doing read back authentication\n");
            break;

         case MD_VERIFY_GONE:
            check_present = FALSE;
            if (!RovingIsPresent(ts))
            {
               reset_screen();  // ADDED for PALM
               debit_mode = MD_FIND_ROVING;
               sprintf(msg,"\n** Waiting for roving SHA iButton\n");
            }
            else  
               sprintf(msg,"** Waiting for roving SHA iButton to leave\n");
            break;

         default:
            debit_mode = MD_FIND_ROVING;
            break;

      };

      // error occured so check if roving device present
      if (check_present)
      {
         // loop for up to 500ms waiting for device to return
         tstamp = msGettick() + 500;
         do { } 
         while (!RovingIsPresent(ts) && (tstamp > msGettick()) && !key_abort());
         // if not return then print message
         if (!RovingIsPresent(ts))
         {
            sprintf(msg,"\n>>>>>>>>>>   Please present roving iButton to finish transaction\n\n");
         }
      }

      // send output
      if ((strcmp(lastmsg,msg) != 0) && !key_abort())
      {
         output_status(LV_ALWAYS,msg);
         sprintf(lastmsg,"%s",msg);
      }
      else
         output_status(LV_VERBOSE,".");
   }
   while (!key_abort());

   return EXIT_SUCCESS;
}


//--------------------------------------------------------------------------
// Prints a message, and the current driver versions.  Return TRUE
// driver version were found and FALSE if not.
//
int PrintHeader(char *title)
{
   char msg[512]; 
   int msgcnt = 0,rt=1;

   // print a header with driver version info   
   sprintf(msg,"\n%s\n",title);
   output_status(LV_ALWAYS,msg);
   output_status(LV_ALWAYS,"Dallas Semiconductor Corporation\n");
   output_status(LV_ALWAYS,"Copyright (C) 1992-2000\n\n");
   output_status(LV_ALWAYS,"Click on screen to end program\n\n");
   
   return rt;
}




