//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//--------------------------------------------------------------------------
//
//  Starter.C - The main subroutine call that initializes all of the calls.

//#include <Pilot.h>
#include "ownet.h"
#include <SysEvtMgr.h>
#include <Hardware.h>
#include <string.h>
#include "common.h"
#include "sysall.h"
#include "StarterRsc.h"
//#include "ownet.h"
#include "SauthPalm.h"
#include "thermo21.h"


// External Functions	
extern int  owAcquire(int,char *, char *);
extern void owRelease(int,char *);
extern int RetPort(int);
extern int FindDevices(int,uchar FamilySN[][8],int,int);
extern int ReadThermoStatus(int,uchar *,ThermoStateType *, FILE *);
extern void  InterpretStatus(MissionStatus *);
extern void MissionStatusToString(MissionStatus *,int,char *);
extern int  owSpeed(int,int);
extern void reset_screen(void);
extern void output_status(int, char *);

// local funcitons
static Word StartApplication(void);

// Local Variables
#define MAXDEVICES 	5
uchar family1[MAXDEVICES][8];
uchar family2[MAXDEVICES][8];
ThermoStateType ThermoState;

DWord PilotMain(Word cmd, char *cmdPBP, Word launchflags)
{
   	Word error;
   
	char return_msg[128];
	EventType event;
	int i,j,numdev1,numdev2;
	int notclear = TRUE;
	uchar tempfam[8];
	int ConvertToF = TRUE;
	char str[558];
	int portnum1 = 0;
	int portnum2 = 1;
	char port1[20],port2[20];
	FILE temp;
	
	sprintf(port1,"%s","0");
	sprintf(port2,"%s","1");

   	if(cmd == sysAppLaunchCmdNormalLaunch)
   	{
      	error = StartApplication();
      	if(error) return error;
      	
   		if (!owAcquire(portnum2,port2,return_msg))
   		{  
   			WinDrawChars((char *) "Could not find One-Wire Network",31,10,110);
   			WinDrawChars((char *) "Now Exiting",11,10,120);
   			SysTaskDelay(500);   			
      		return 0;
   		}
   		else
   		{
   			if(owAcquire(portnum1,port1,return_msg))
   			{
   			  do
   			  {
   			    owSpeed(portnum1,MODE_NORMAL);
   				numdev1 = FindDevices(portnum1,&family1[0],THERMO_FAM,MAXDEVICES);
   				numdev2 = FindDevices(portnum2,&family2[0],THERMO_FAM,MAXDEVICES);
   			   			
   				if((numdev1 == 0) && (numdev2 == 0))
   				{
   					if(notclear)
   					{
   						reset_screen();
   						notclear = FALSE;
   					}
   						
   					WinDrawChars((char *) "Thermochron not present on 1-Wire",33,10,10);
					WinDrawChars((char *) "PRESS TO EXIT OR ADD DS1921",27,10,20);
   				}
   				else
   				{
   					notclear = TRUE;
   					
   					for(i=0;i<numdev1;i++)
   					{
   						reset_screen();
   						for (j = 7; j >= 0; j--)
   						{
         					ThermoState.MissStat.serial_num[j] = family1[i][j];
         					tempfam[j] = family1[i][j];
         				}
	
						iBSetup(RetPort(portnum1));
						if(ReadThermoStatus(portnum1, &family1[i][0], &ThermoState, &temp))
						{
							InterpretStatus(&ThermoState.MissStat);

							MissionStatusToString(&ThermoState.MissStat, ConvertToF, &str[0]);
							
							output_status(LV_ALWAYS,str);
						}   
						else
							WinDrawChars((char *) "Did not read1",13,5,40);	
								
						sprintf(str,"PRESS TO CONTINUE");
						WinDrawChars(str,StrLen(str),5,140);
						
						do
						{
							EvtGetEvent(&event,1);
						} while(event.eType != penDownEvent);						
					}	

   					for(i=0;i<numdev2;i++)
   					{
   						reset_screen();
   						for (j = 7; j >= 0; j--)
   						{
         					ThermoState.MissStat.serial_num[j] = family2[i][j];
         					tempfam[j] = family2[i][j];
         				}
	
						iBSetup(RetPort(portnum2));
						if(ReadThermoStatus(portnum2, &family2[i][0], &ThermoState, &temp))
						{
							InterpretStatus(&ThermoState.MissStat);

							MissionStatusToString(&ThermoState.MissStat, ConvertToF, &str[0]);
							
							output_status(LV_ALWAYS,str);
						}   
						else
							WinDrawChars((char *) "Did not read2",13,10,40);		

						WinDrawChars((char *) "PRESS TO CONTINUE",17,5,140);
												
						do
						{
							EvtGetEvent(&event,1);
						} while(event.eType != penDownEvent);						
					}	

				  } 
				     			
      			EvtGetEvent(&event,1);
      		
      		} while(event.eType != penDownEvent);

			// release the 1-Wire Net	  
	  		owRelease(portnum1, return_msg);
	  		owRelease(portnum2, return_msg);
	  		 
	  	  }//Acquire1
	  	  else
	  	  {
	  	  	output_status(LV_ALWAYS,"Couldn't find second port\n");
	  		owRelease(portnum2, return_msg);
	  	  }
		}
		
	}   
	     
    return 0;   
}

//---------------------------------------------------------------------
static Word StartApplication(void)
{
   FrmGotoForm(MainForm);  
   return 0;
}


